@@ -7,6 +7,7 @@
 #               Kayo Ide                        U. of Maryland
 #               David Santek                    CIMSS at U. of Wisconsin-Madison
 #               Brett Hoover                    NOAA/NWS/NCEP/EMC, Lynker Technologies
+#               David Huber                     NOAA/OAR/WPO/EPIC, Redline Performance Solutions, LLC
 #               Ross N. Hoffman                 NOAA/NESDIS/STAR, CISESS at U. of Maryland
 #               Hui Liu                         NOAA/NESDIS/STAR, CISESS at U. of Maryland
 #
@@ -16,12 +17,13 @@
 # Modified by:  Katherine E. Lukens
 #
 # HISTORY: 
-#	2021		B. Hoover, D. Santek	Created/developed program.
+#	2021		B. Hoover, D. Santek    Created/developed program.
 #	2021-10-25	K.E. Lukens		Added aircraft, AMV input configs. Added quality controls (QC).
 #	2021-11-22	K.E. Lukens		Added date before consideration.
 #	2021-11-23	K.E. Lukens		Created QC module and added relevant calls.
 #	2022-03-08	K.E. Lukens		Added metadata to output files.
-#	2023-04-14	K.E. Lukens		Finalized for upload to SAWC archive.
+#	2023-03-29	K.E. Lukens		Finalized for upload to wind-datasets archive.
+#	2023-07-15	D. Huber, K.E. Lukens	Optimized program to improve performance.
 #
 # Usage: python3 ThisScriptName.py $driver_wind_type $TF_drv_qc $TF_dset2_qc $pct $dateIN $driver_dset_type_str
 #
@@ -72,7 +74,7 @@ from read_data import read_aeolus
 from read_data import read_aircraft
 from read_data import read_amv_ncep
 from read_data import read_loon
-from read_data import read_raobs
+from read_data import read_sonde
 
 ##############################################
 
@@ -338,8 +340,8 @@ elif driver_name=='AMV_NCEP':
   drv_lat,drv_lon,drv_yr,drv_mm,drv_dy,drv_hr,drv_mn,drv_hgt,drv_prs,indexesD,qc_drv_list,drv_src,drv_satname,drv_wcm,drv_ham,drv_spd,drv_dir = read_amv_ncep(archive_parent,yyyymmddhh,dateB4,dateA,bool_drv_qc,pct_drv,qi_choice_drv,"drv",runtype,tim_max_str,idxs)
 elif driver_name=='Loon':
   drv_lat,drv_lon,drv_yr,drv_mm,drv_dy,drv_hr,drv_mn,drv_hgt,drv_prs,indexesD,qc_drv_list,drv_src,drv_hgt,drv_azm,drv_elv,drv_spd,drv_dir = read_loon(archive_parent,yyyymmddhh,dateB4,dateA,bool_drv_qc,"drv",runtype,tim_max_str,idxs)
-elif driver_name=='Radiosonde':
-  drv_lat,drv_lon,drv_yr,drv_mm,drv_dy,drv_hr,drv_mn,drv_hgt,drv_prs,indexesD,qc_drv_list,drv_src,nsondes,nlevels,ngroups,drv_spd,drv_dir = read_raobs(archive_parent,yyyymmddhh,dateB4,dateA,bool_drv_qc,"drv",runtype,tim_max_str,idxs)
+elif driver_name=='Sonde':
+  drv_lat,drv_lon,drv_yr,drv_mm,drv_dy,drv_hr,drv_mn,drv_hgt,drv_prs,indexesD,qc_drv_list,drv_src,nsondes,nlevels,ngroups,drv_spd,drv_dir = read_sonde(archive_parent,yyyymmddhh,dateB4,dateA,bool_drv_qc,"drv",runtype,tim_max_str,idxs)
 else:
   print("ERROR: Driver dataset not defined!")
   sys.exit()
@@ -380,12 +382,12 @@ for i in range(ndependent_datasets):
     t_lat,t_lon,t_yr,t_mm,t_dy,t_hr,t_mn,t_hgt,t_prs,t_indexes,t_qc_dset_list,t_src,t_satname,t_wcm,t_ham,t_spd,t_dir = read_amv_ncep(archive_parent,yyyymmddhh,dateB4,dateA,bool_dset_qc[i],pct[i],qi_choice[i],"dep",runtype,tim_max_str,idxs)
   elif dependent_names[i]=='Loon':
     t_lat,t_lon,t_yr,t_mm,t_dy,t_hr,t_mn,t_hgt,t_prs,t_indexes,t_qc_dset_list,t_src,t_azm,t_elv,t_spd,t_dir = read_loon(archive_parent,yyyymmddhh,dateB4,dateA,bool_dset_qc[i],"dep",runtype,tim_max_str,idxs)
-  elif dependent_names[i]=='Radiosonde':
+  elif dependent_names[i]=='Sonde':
 	# RADIOSONDE data are 2D (except for lat, lon, dates/times): [nsondes, nlevels]
 	# 	nsondes = number of sondes
 	# 	nlevels = number of vertical levels per sonde
 	# nsondes, nlevels are used to find collocated indices that pertain to original data file
-    t_lat,t_lon,t_yr,t_mm,t_dy,t_hr,t_mn,t_hgt,t_prs,t_indexes,t_qc_dset_list,t_src,nsondes,nlevels,ngroups,t_spd,t_dir = read_raobs(archive_parent,yyyymmddhh,dateB4,dateA,bool_dset_qc[i],"dep",runtype,tim_max_str,idxs)
+    t_lat,t_lon,t_yr,t_mm,t_dy,t_hr,t_mn,t_hgt,t_prs,t_indexes,t_qc_dset_list,t_src,nsondes,nlevels,ngroups,t_spd,t_dir = read_sonde(archive_parent,yyyymmddhh,dateB4,dateA,bool_dset_qc[i],"dep",runtype,tim_max_str,idxs)
   else:
     print("ERROR: Dependent dataset "+str(i)+" ("+dependent_names[i]+") cannot be read by this program! Please add function to read_input_datasets module and try again.")
     sys.exit()
@@ -821,28 +823,28 @@ if __name__ ==  '__main__':
       var_miss  += [str(fill)]
 		
 	#------------------------------------------
-	# This code block is for profile datasets with 2D variables (e.g., Radiosonde)
+	# This code block is for profile datasets with 2D variables (e.g., Sonde)
 
-      if driver_name == "Radiosonde" or dependent_names[ndsets] == "Radiosonde":
+      if driver_name == "Sonde" or dependent_names[ndsets] == "Sonde":
         var_vals  += [ngroups]  #append number of sondes per group to NetCDF output var
         var_names += ['ntypes_dset'+str(ndsets+1)]
-        var_longnames += ['Number of Radiosonde types']
+        var_longnames += ['Number of Sonde types']
         var_units += [str(fill)]
         var_miss  += [str(fill)]
 
         var_vals  += [nsondes]	#append number of sondes per group to NetCDF output var
         var_names += ['nprofiles_dset'+str(ndsets+1)]
-        var_longnames += ['Number of Radiosondes per type'] 
+        var_longnames += ['Number of Sondes per type'] 
         var_units += [str(fill)]
         var_miss  += [str(fill)]
 
         var_vals  += [nlevels]	#append number of levels per sonde per group to NetCDF output var
         var_names += ['nlevels_dset'+str(ndsets+1)]
-        var_longnames += ['Number of max vertical levels for each Radiosonde per type'] 
+        var_longnames += ['Number of max vertical levels for each Sonde per type'] 
         var_units += [str(fill)]
         var_miss  += [str(fill)]
 
-		#NOTE: plotting script will use these variables to extract indices of Radiosonde obs collocated with DRIVER
+		#NOTE: plotting script will use these variables to extract indices of Sonde obs collocated with DRIVER
 
 	#------------------------------------------
 
@@ -872,7 +874,7 @@ if __name__ ==  '__main__':
     
       #==============================================================
     
-print("***** END MAIN PROGRAM *****")
+print("===== END MAIN PROGRAM =====")
 #########################################################################################################
 # END SCRIPT
 #########################################################################################################
