@@ -7,6 +7,7 @@
 #               Kayo Ide                        U. of Maryland
 #               David Santek                    CIMSS at U. of Wisconsin-Madison
 #               Brett Hoover                    NOAA/NWS/NCEP/EMC, Lynker Technologies
+#               David Huber                     NOAA/OAR/WPO/EPIC, Redline Performance Solutions, LLC
 #               Ross N. Hoffman                 NOAA/NESDIS/STAR, CISESS at U. of Maryland
 #               Hui Liu                         NOAA/NESDIS/STAR, CISESS at U. of Maryland
 #
@@ -18,6 +19,7 @@
 # History:
 #       2021            B. Hoover, D. Santek    Created/developed program.
 #       2023-04-14      K.E. Lukens             Finalized for upload to SAWC archive.
+#	2023-07-15	D. Huber, K.E. Lukens	Optimized program to improve performance.
 #
 # Built with the following conda environment:
 #
@@ -308,7 +310,6 @@ def match_obs(
 	       hgtd  ,   		# height .......... dask-array
 	       hgtn  ,   		# height drv ...... numpy-array
 	       thgt_max ,   		# maximum height-difference (units=km) 
-               dset_choice,		# indicates which dataset to output here: 
 					#	'matches'=matched indices, 
 					#	'dependents'=integer values pertaining to each dependent dataset per matched index 
                parallelize_set_1 ,	# flag indicating which dataset is the dask-array (parallelized) (False=DRIVER, or True=DEPENDENTS)  
@@ -316,12 +317,13 @@ def match_obs(
              ):
 
 	# Create output arrays
-    matches = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)  #... List of numpy indices matching dask ob (initialized to nan)
-    GCDs    = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)  #... List of great circle distances for matching dask ob (initialized to nan)
-    DPs     = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)  #... List of pressure differences for matching dask ob (initialized to nan)
-    DPslog  = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)  #... List of log10(pressure) differences for matching dask ob (initialized to nan)
-    HTs     = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)  #... List of height differences for matching dask ob (initialized to nan)
-    DTs     = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)  #... List of time differences for matching dask ob (initialized to nan)
+    results = np.nan * np.ones((np.size(latd),n_max,6),dtype=latd.dtype)  #... List of matching indices, dim size = 6 (for the following 6 vars) (initialized to nan)
+    matches = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)    #... List of numpy indices matching dask ob (initialized to nan)
+    GCDs    = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)    #... List of great circle distances for matching dask ob (initialized to nan)
+    DPs     = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)    #... List of pressure differences for matching dask ob (initialized to nan)
+    DPslog  = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)    #... List of log10(pressure) differences for matching dask ob (initialized to nan)
+    HTs     = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)    #... List of height differences for matching dask ob (initialized to nan)
+    DTs     = np.nan * np.ones((np.size(latd),n_max),dtype=latd.dtype)    #... List of time differences for matching dask ob (initialized to nan)
 
     	# Loop through dask array (dependehap datasets)
     for i in range(np.size(latd)):
@@ -506,20 +508,10 @@ def match_obs(
             DPslog[i,k] = mDPlog[k]
             HTs[i,k] = mHT[k]
             DTs[i,k] = mDT[k]
+		# 'results' contains the above 6 arrays for each i
+            results[i,k,0:6] = [m[k], mGCD[k], mDP[k], mDPlog[k], mHT[k], mDT[k]]
 
-    if dset_choice=='matches':
-      results = matches
-    elif dset_choice=='GCDs':
-      results = GCDs
-    elif dset_choice=='DPs':
-      results = DPs
-    elif dset_choice=='DPslog':
-      results = DPslog
-    elif dset_choice=='HTs':
-      results = HTs
-    elif dset_choice=='DTs':
-      results = DTs
-
+    #print("Did we make it to return?", results.shape)
     return results
 #
 ###########################################################################
@@ -792,10 +784,10 @@ def obs_match_3d(
     #
     #
     # Apply matching function via da.map_blocks
+    #	'result' contains 6 arrays returned by 'match_obs' function
     #
-    print("CALL match_obs for matched indices")
-    dset_choice = 'matches'
-    matches = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
+    print("CALL match_obs")
+    result = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
                              match_obs , 
                              lat_da    , 
                              lon_da    , 
@@ -820,167 +812,6 @@ def obs_match_3d(
 			     hgt_da    ,
 			     hgt_np    ,
 			     hgt_max_npda   ,
-                             dset_choice,
-                             parallelize_set_1,
-                             usePH_chunk
-                           )
-
-    print("CALL match_obs for matched Great Circle Distances (GCDs)")
-    dset_choice = 'GCDs'
-    GCDs = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
-                             match_obs ,
-                             lat_da    ,
-                             lon_da    ,
-                             prs_da    ,
-                             yr_da     ,
-                             mm_da     ,
-                             dy_da     ,
-                             hr_da     ,
-                             mn_da     ,
-                             lat_np    ,
-                             lon_np    ,
-                             prs_np    ,
-                             yr_np     ,
-                             mm_np     ,
-                             dy_np     ,
-                             hr_np     ,
-                             mn_np     ,
-                             dst_max_npda   ,
-                             prs_max_npda   ,
-                             tim_max_npda   ,
-                             n_max     ,
-                             hgt_da    ,
-                             hgt_np    ,
-                             hgt_max_npda   ,
-                             dset_choice,
-                             parallelize_set_1,
-                             usePH_chunk
-                           )
-
-    print("CALL match_obs for matched pressure differences (DPs)")
-    dset_choice = 'DPs'
-    DPs = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
-                             match_obs ,
-                             lat_da    ,
-                             lon_da    ,
-                             prs_da    ,
-                             yr_da     ,
-                             mm_da     ,
-                             dy_da     ,
-                             hr_da     ,
-                             mn_da     ,
-                             lat_np    ,
-                             lon_np    ,
-                             prs_np    ,
-                             yr_np     ,
-                             mm_np     ,
-                             dy_np     ,
-                             hr_np     ,
-                             mn_np     ,
-                             dst_max_npda   ,
-                             prs_max_npda   ,
-                             tim_max_npda   ,
-                             n_max     ,
-                             hgt_da    ,
-                             hgt_np    ,
-                             hgt_max_npda   ,
-                             dset_choice,
-                             parallelize_set_1,
-                             usePH_chunk
-                           )
-
-    print("CALL match_obs for matched log10(p) differences (DPslog)")
-    dset_choice = 'DPslog'
-    DPslog = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
-                             match_obs ,
-                             lat_da    ,
-                             lon_da    ,
-                             prs_da    ,
-                             yr_da     ,
-                             mm_da     ,
-                             dy_da     ,
-                             hr_da     ,
-                             mn_da     ,
-                             lat_np    ,
-                             lon_np    ,
-                             prs_np    ,
-                             yr_np     ,
-                             mm_np     ,
-                             dy_np     ,
-                             hr_np     ,
-                             mn_np     ,
-                             dst_max_npda   ,
-                             prs_max_npda   ,
-                             tim_max_npda   ,
-                             n_max     ,
-                             hgt_da    ,
-                             hgt_np    ,
-                             hgt_max_npda   ,
-                             dset_choice,
-                             parallelize_set_1,
-                             usePH_chunk
-                           )
-    
-    print("CALL match_obs for matched height differences (HTs)")
-    dset_choice = 'HTs'
-    HTs = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
-                             match_obs ,
-                             lat_da    ,
-                             lon_da    ,
-                             prs_da    ,
-                             yr_da     ,
-                             mm_da     ,
-                             dy_da     ,
-                             hr_da     ,
-                             mn_da     ,
-                             lat_np    ,
-                             lon_np    ,
-                             prs_np    ,
-                             yr_np     ,
-                             mm_np     ,
-                             dy_np     ,
-                             hr_np     ,
-                             mn_np     ,
-                             dst_max_npda   ,
-                             prs_max_npda   ,
-                             tim_max_npda   ,
-                             n_max     ,
-                             hgt_da    ,
-                             hgt_np    ,
-                             hgt_max_npda   ,
-                             dset_choice,
-                             parallelize_set_1,
-                             usePH_chunk
-                           )
-
-    print("CALL match_obs for matched time differences")
-    dset_choice = 'DTs'
-    DTs = da.map_blocks( #............................................. Matched numpy indices (uncomputed)
-                             match_obs ,
-                             lat_da    ,
-                             lon_da    ,
-                             prs_da    ,
-                             yr_da     ,
-                             mm_da     ,
-                             dy_da     ,
-                             hr_da     ,
-                             mn_da     ,
-                             lat_np    ,
-                             lon_np    ,
-                             prs_np    ,
-                             yr_np     ,
-                             mm_np     ,
-                             dy_np     ,
-                             hr_np     ,
-                             mn_np     ,
-                             dst_max_npda   ,
-                             prs_max_npda   ,
-                             tim_max_npda   ,
-                             n_max     ,
-                             hgt_da    ,
-                             hgt_np    ,
-                             hgt_max_npda   ,
-                             dset_choice,
                              parallelize_set_1,
                              usePH_chunk
                            )
@@ -998,41 +829,18 @@ def obs_match_3d(
     #    numpy arrays as a one-chunk array.
     #
     clock1 = time.time() #................................................. Beginning-time of task
-    ob_idx_np=matches.compute(num_workers=nproc) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
+    results_np=result.compute(num_workers=1) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
     clock2 = time.time() #................................................. Ending-time of task
     clock = clock2-clock1
     print('Match computing completed in ',clock,'seconds')
 
-	# Dask array computes for GCD, DP, DT
-    clock1 = time.time() #................................................. Beginning-time of task
-    GCD_np=GCDs.compute(num_workers=nproc) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
-    clock2 = time.time() #................................................. Ending-time of task
-    clock = clock2-clock1
-    print('GCD Match computing completed in ',clock,'seconds')
-
-    clock1 = time.time() #................................................. Beginning-time of task
-    DP_np=DPs.compute(num_workers=nproc) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
-    clock2 = time.time() #................................................. Ending-time of task
-    clock = clock2-clock1
-    print('DP Match computing completed in ',clock,'seconds')
-
-    clock1 = time.time() #................................................. Beginning-time of task
-    DPlog_np=DPslog.compute(num_workers=nproc) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
-    clock2 = time.time() #................................................. Ending-time of task
-    clock = clock2-clock1
-    print('DP log10 Match computing completed in ',clock,'seconds')
-    
-    clock1 = time.time() #................................................. Beginning-time of task
-    HT_np=HTs.compute(num_workers=nproc) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
-    clock2 = time.time() #................................................. Ending-time of task
-    clock = clock2-clock1
-    print('HT Match computing completed in ',clock,'seconds')
-
-    clock1 = time.time() #................................................. Beginning-time of task
-    DT_np=DTs.compute(num_workers=nproc) #......................... Matched indices of numpy array for each ob in dask array (numPy array)
-    clock2 = time.time() #................................................. Ending-time of task
-    clock = clock2-clock1
-    print('DT Match computing completed in ',clock,'seconds')
+	# separate 'results_np' (from 'result') into each of its 6 arrays
+    ob_idx_np = results_np[:,:,0]
+    GCD_np = results_np[:,:,1]
+    DP_np = results_np[:,:,2]
+    DPlog_np = results_np[:,:,3]
+    HT_np = results_np[:,:,4]
+    DT_np = results_np[:,:,5]
 
     #######################################################################
     #
@@ -1134,7 +942,7 @@ def match_lists_to_netCDF(names,vals,nc_out_filenm,longnames,units,missing,src_p
 
     # Add Data
     n_vars = len(names) #.................................................. Number of variables to output
-    print("=== START LOOP ===")
+    #print("=== START LOOP ===")
     for i in range(n_vars):
 	
       # Set dimension name
