@@ -7,15 +7,13 @@
 #               Kayo Ide                        U. of Maryland
 #               David Santek                    CIMSS at U. of Wisconsin-Madison
 #               Brett Hoover                    NOAA/NWS/NCEP/EMC, Lynker Technologies
+#               David Huber                     NOAA/OAR/WPO/EPIC, Redline Performance Solutions, LLC
 #               Ross N. Hoffman                 NOAA/NESDIS/STAR, CISESS at U. of Maryland
 #               Hui Liu                         NOAA/NESDIS/STAR, CISESS at U. of Maryland
 #
-# History:
-#       2022            K.E. Lukens	        Created/developed program.
-#       2023-04-14      K.E. Lukens             Finalized for upload to SAWC archive.
-#
 # Built with the following conda environment:
 #
+# name: bhoover-obs_match_3d
 # channels:
 #   - conda-forge
 #   - defaults
@@ -87,7 +85,7 @@ def qc_aeolus(driver_type,prs,err,length,htop,hbot):
     if driver_type == 'MieCloud':
 	# QC limits
         pmax            = 800.          #max pressure allowed. Units = hPa
-        emax_p          = 5.            #max error at lower levels. Units = m/s
+        emax_p          = 5.            #max error. Units = m/s
 
         # Indices where dataset PASSES QC
         #       Using np.where: To get new array size where input conditions are True,
@@ -174,5 +172,32 @@ def qc_winds(sdrv_spd,sdset_spd):
 
     return idx
 
+# -------------------------------------------------------------------------
+# QC for Wind Speed
+#       Applies to one dataset (prior to matching)
+#
+#       INPUTS:
+#               pccf ................................ percent confidence (quality indicator, QI) in data point
+#               pct ................................. min percent allowed
+#
+#       OUTPUTS:
+#               idx ................................. indices where dataset PASSES QC
+#
+def qc_winds_each(sdrv_spd):
+    # Gross check for wind speed: omit missing values and unrealistic winds (e.g., winds larger than spd_max)
+
+    # Max speed difference allowed between 'drv' and 'dset'
+    spd_max = 25.0              # units = m/s
+
+    drv_spd  = np.asarray(sdrv_spd)
+    del sdrv_spd
+
+    # Indices where datasets PASS QC:
+    #   Where speed is not NaN
+    #   Where speed differences < 'spd_max'
+    idx = np.where((drv_spd != np.nan)*(abs(drv_spd) < spd_max))
+
+    return idx
+
 
 ###########################################################################
